<?php
/**
 * Event class
 *
 * PHP version 5
 *
 * @category ADPOI
 * @package  Core.model
 * @author   Evgeniy Marilev <jeka@5-soft.com>
 * @license  http://adpoi.com/display/license proprietary
 * @link     http://adpoi.com/display/PhpDoc
 */
/**
 * Event is the model class for table "event".
 * The followings are the available columns in table 'event':
 *
 * @property integer $id
 * @property integer $poi_id
 * @property integer $creator_id
 * @property string  $name
 * @property string  $create_time
 * @property string  $start_time
 * @property string  $end_time
 * @property string  $description
 * @property string  $state
 * 
 * @category ADPOI
 * @package  Core.model
 * @author   Evgeniy Marilev <jeka@5-soft.com>
 * @license  http://adpoi.com/display/license proprietary
 * @link     http://adpoi.com/display/PhpDoc
 */
class Event extends CActiveRecord implements IOrganizerEvent
{
    const STATE_OPENED = 'opened';
    const STATE_CLOSED = 'closed';
    
    /**
     * Returns the static model of the specified AR class.
     * 
     * @param string $className the name of the model class
     * 
     * @return Event the static model class
     */
    public static function model($className = __CLASS__)
    {
        return parent::model($className);
    }

    /**
     * Returns associated table name
     * 
     * @return string the associated database table name
     */
    public function tableName()
    {
        return 'event';
    }

    /**
     * Returns validation rules
     * 
     * @return array validation rules for model attributes.
     */
    public function rules()
    {
        return array(
            array('poi_id, creator_id, name', 'required'),
            array('poi_id, creator_id', 'numerical', 'integerOnly'=>true),
            array('name', 'length', 'max'=>255),
            array('description', 'length', 'max'=>1024),
            array('state', 'in', 'range' => self::getStateTypes()),
            array('start_time, end_time', 'safe'),
            array('id, poi_id, creator_id, name, create_time, start_time, description, state', 'safe', 'on'=>'search'),
        );
    }

    /**
     * Returns model's relations
     * 
     * @return array relational rules.
     */
    public function relations()
    {
        return array(
            'poi' => array(self::BELONGS_TO, 'Poi', 'poi_id'),
            'creator' => array(self::BELONGS_TO, 'User', 'creator_id'),
            'users' => array(self::MANY_MANY, 'User', 'event_staff(event_id, user_id)'),
            'organizer_event' => array(self::HAS_MANY, 'OrganizerEvent', 'event_id')
        );
    }

    /**
     * Returns attributes labels
     * 
     * @return array customized attribute labels (name=>label)
     */
    public function attributeLabels()
    {
        return array(
            'id' => Yii::t('app', 'Id'),
            'poi_id' => Yii::t('app', 'Poi'),
            'creator_id' => Yii::t('app', 'Creator'),
            'name' => Yii::t('app', 'Name'),
            'create_time' => Yii::t('app', 'Create Time'),
            'start_time' => Yii::t('app', 'Start Time'),
            'description' => Yii::t('app', 'Description'),
            'state' => Yii::t('app', 'State'),
        );
    }

    /**
     * Retrieves a list of models based on the current search/filter conditions.
     * 
     * @return CActiveDataProvider the data provider that can return the models based on the search/filter conditions.
     */
    public function search()
    {
        $criteria = new CDbCriteria;

        $criteria->compare('id', $this->id);

        $criteria->compare('poi_id', $this->poi_id);

        $criteria->compare('creator_id', $this->creator_id);

        $criteria->compare('name', $this->name, true);

        $criteria->compare('create_time', $this->create_time, true);

        $criteria->compare('start_time', $this->start_time, true);
        
        $criteria->compare('end_time', $this->end_time, true);

        $criteria->compare('description', $this->description, true);

        $criteria->compare('state', $this->state, true);

        return new CActiveDataProvider('Event', array(
            'criteria' => $criteria,
        ));
    }
    
    /**
     * Returns event identifier
     * 
     * @return string event identifier
     * @see IOrganizerEvent::getIdentifier()
     */
    public function getIdentifier()
    {
        return $this->getAttribute('id');
    }
    
    /**
     * Returns event name
     * 
     * @return string event name
     * @see IOrganizerEvent::getName()
     */
    public function getName()
    {
        return $this->getAttribute('name');
    }
    
    /**
     * Returns event start time
     * 
     * @return string event start time
     * @see IOrganizerEvent::getStartTime()
     */
    public function getStartTime()
    {
        return $this->getAttribute('start_time');
    }
    
    /**
     * Returns event end time
     * 
     * @return string event end time
     * @see IOrganizerEvent::getEndTime()
     */
    public function getEndTime()
    {
        return $this->getAttribute('end_time');
    }
    
    /**
     * Returns event description
     * 
     * @return string event description
     * @see IOrganizerEvent::getDescription()
     */
    public function getDescription()
    {
        return $this->getAttribute('description');
    }
    
    /**
     * Returns event location place
     * 
     * @return string event location place
     * @see IOrganizerEvent::getLocation()
     */
    public function getLocation()
    {
        return $this->poi->name;
    }
    
    /**
     * Close event
     * 
     *  @return bool the is operation success
     */
    public function close()
    {
        $this->state = self::STATE_CLOSED;
        return $this->save();
    }
    
    /**
     * Return the state possible values
     * 
     * @return array the state possible values
     */
    public static function getStateTypes()
    {
        return array(
            self::STATE_OPENED,
            self::STATE_CLOSED,
        );
    }
}