<?php
/**
 * EventApiModel class
 *
 * PHP version 5
 *
 * @category ADPOI
 * @package  Api.controller
 * @author   Pohil Alexandr <pohil@5-soft.com>
 * @license  http://adpoi.com/display/license proprietary
 * @link     http://adpoi.com/display/PhpDoc
 */
/**
 * EventApiModel is the model class
 * for managing Event records
 *
 * @category ADPOI
 * @package  Api.controller
 * @author   Pohil Alexandr <pohil@5-soft.com>
 * @license  http://adpoi.com/display/license proprietary
 * @link     http://adpoi.com/display/PhpDoc
 */
class EventApiModel extends RestApiModel
{
    const DATE_FORMAT = 'yyyy-MM-dd HH:mm:ss';
    
    /**
     * Event's id
     * @var integer
     */
    public $id;
    
    /**
     * Poi's id
     * @var integer
     */
    public $poi_id;

    /**
     * Creator's id
     * @var integer
     */
    public $creator_id;
    
    /**
     * Events' name
     * @var string
     */
    
    public $name;
    
    /**
     * Event's start time
     * @var datetime
     */
    public $start_time;
    
    /**
     * Event's end time
     * @var datetime
     */
    public $end_time;
    
    /**
     * Event's top border of start time
     * @var datetime
     */
    public $to;
    
    /**
     * Event's lower border of start time
     * @var datetime
     */
    public $from;
    
    /**
     * Event's description
     * @var string
     */
    public $description;
        
    /**
     * Event's state name
     * @var string
     */
    public $state;
    
    /**
     * Count of events on page
     * @var integer
     */
    public $count;
    
    /**
     * Number of event's page
     * @var integer
     */
    public $page;
    
    /**
     * Return the validation rulles
     * 
     * @return array validation rules to be applied 
     * when {@link validate()} is called.
     * @see CModel::rules()
     */
    public function rules()
    {
        $base = parent::rules();
        $rules = array(
            array('id', 'required', 'on' => 'update, delete, show'),
            array('id', 'numerical', 'integerOnly' => true, 'on' => 'update, delete, show'),
            array('id', 'exist', 'className' => 'Event', 'attributeName' => 'id', 'on' => 'update, delete, show'),
            array('poi_id', 'required', 'on' => 'create'),
            array('poi_id', 'numerical',  'integerOnly' => true, 'on' => 'create'),
            array('poi_id', 'exist', 'className' => 'Poi', 'attributeName' => 'id', 'on' => 'create'),
            array('name, start_time, end_time', 'required', 'on' => 'create'),
            array('name', 'length', 'max'=>255, 'on' => 'create, update'),
            array('start_time, end_time', 'date', 'format' => self::DATE_FORMAT, 'on' => 'create, update'),
            array('to, from', 'required', 'on' => 'list'),
            array('to, from', 'date', 'format' => self::DATE_FORMAT, 'on' => 'list'),
            array('description', 'length', 'max'=>1024, 'on' => 'create, update'),
            array('start_time', 'compare', 'compareValue' => 'end_time', 'operator' => '<', 'on' => 'create, update'),
            array('count, page', 'numerical', 'integerOnly' => true, 'min' => 1, 'on' => 'list'),
            array('page', 'default', 'value' => 1, 'on' => 'list'),
            array('count', 'default', 'value' => 10, 'on' => 'list'),
        );
        return CMap::mergeArray($base, $rules);
    }
    
    /**
     * Submits changes into database for create operations
     *  
     * @return bool is operation success 
     * @see RestApiModel::onCreate()
     */
    protected function onCreate()
    {
        $event = $this->loadRecord();
        $event->attributes = $this->attributes;
        $organizer = Yii::app()->organizer;
        $organizer->setUser($event->creator);
        return $organizer->createEvent($event);
    }
    
    /**
     * Submits changes into database for update operations
     *  
     * @return bool is operation success 
     * @see RestApiModel::onUpdate()
     */
    protected function onUpdate()
    {
        $event = $this->loadRecord();
        $event->attributes = $this->attributes;
        $organizer = Yii::app()->organizer;
        $organizer->setUser($event->creator);
        return $organizer->updateEvent($event);
    }
    
    /**
     * Submits changes into database for delete operations
     *  
     * @return bool is operation success 
     * @see RestApiModel::onDelete()
     */
    protected function onDelete()
    {
        $event = $this->loadRecord();
        $organizer = Yii::app()->organizer;
        $organizer->setUser($event->creator);
        return $organizer->deleteEvent($event);
    }
    
    
    /**
     * Fetchs api model's records list.
     * Used only for "list" API method
     * 
     * @return array records list
     */
    protected function fetchData()
    {
        $criteria = new CDbCriteria;
        $criteria->condition = 'start_time BETWEEN :from AND :to';
        $criteria->params = array(
            ':to' => $this->to,
            ':from' => $this->from
        );
        $criteria->limit = $this->count;
        $criteria->offset = ($this->page - 1) * $this->count;
        return Event::model()->with('poi', 'creator')->findAll($criteria);
    }
}