<?php
/**
 * Poi class
 *
 * PHP version 5
 *
 * @category ADPOI
 * @package  Core.model
 * @author   Evgeniy Marilev <jeka@5-soft.com>
 * @license  http://adpoi.com/display/license proprietary
 * @link     http://adpoi.com/display/PhpDoc
 */
/**
 * Poi is the model class for table "poi".
 * The followings are the available columns in table 'poi':
 *
 * @property integer $id
 * @property string  $google_id
 * @property integer $google_rating
 * @property string  $name
 * @property string  $description
 * @property double  $latitude
 * @property double  $longitude
 * @property string  $phone
 * @property string  $email
 * @property string  $skype
 * @property integer $rating
 * 
 * @category ADPOI
 * @package  Core.model
 * @author   Evgeniy Marilev <jeka@5-soft.com>
 * @license  http://adpoi.com/display/license proprietary
 * @link     http://adpoi.com/display/PhpDoc
 */
class Poi extends CActiveRecord
{
    /**
     * Returns the static model of the specified AR class.
     * 
     * @param string $className the name of the model class
     * 
     * @return Poi the static model class
     */
    public static function model($className = __CLASS__)
    {
        return parent::model($className);
    }

    /**
     * Returns associated table name
     * 
     * @return string the associated database table name
     */
    public function tableName()
    {
        return 'poi';
    }

    /**
     * Returns validation rules
     * 
     * @return array validation rules for model attributes.
     */
    public function rules()
    {
        return array(
            array('name, description, latitude, longitude, phone, email', 'required'),
            array('google_id', 'length', 'max' => 64),
            array('google_rating', 'numerical', 'integerOnly' => true, 'min' => 0),
            array('rating', 'numerical', 'min' => 0),
            array('latitude', 'numerical', 'min' => -90, 'max' => 90),
            array('longitude', 'numerical', 'min' => -180, 'max' => 180),
            array('name', 'length', 'max'=>1024),
            array('phone, skype', 'length', 'max'=>128),
            array('email', 'length', 'max'=>256),
            array('id, google_id, name, description, latitude, longitude, phone, email, skype, rating', 'safe', 'on'=>'search'),
        );
    }

    /**
     * Returns model's relations
     * 
     * @return array relational rules.
     */
    public function relations()
    {
        return array(
            'events' => array(self::HAS_MANY, 'Event', 'poi_id'),
            'eventsCount' => array(self::STAT, 'Event', 'poi_id'),
            'users' => array(self::MANY_MANY, 'User', 'vote(user_id, poi_id)'),
            'categories' => array(self::MANY_MANY, 'Category', 'poi_category(poi_id, category_id)'),
            'poi_categories' => array(self::HAS_MANY, 'PoiCategory', 'poi_id'),
            'favorites' => array(self::HAS_MANY, 'Favorite', 'poi_id'),
            'favoritesCount' => array(self::STAT, 'Favorite', 'poi_id'),
            'photos' => array(self::MANY_MANY, 'Photo', 'poi_photo(photo_id, poi_id)'),
            'photosCount' => array(self::STAT, 'PoiPhoto', 'poi_id'),
            'recommendations' => array(self::HAS_MANY, 'PoiRecommendation', 'poi_id'),
            'recommendationsCount' => array(self::STAT, 'PoiRecommendation', 'poi_id'),
            'votesCount' => array(self::STAT, 'Feedback', 'poi_id',
                'condition' => "type = '" . Feedback::TYPE_VOTE . "'",
            ),
        );
    }

    /**
     * Returns attributes labels
     * 
     * @return array customized attribute labels (name=>label)
     */
    public function attributeLabels()
    {
        return array(
            'id' => Yii::t('app', 'Id'),
            'google_id' => Yii::t('app', 'Google identifier'),
            'google_rating' => Yii::t('app', 'Google rating'),
            'name' => Yii::t('app', 'Name'),
            'description' => Yii::t('app', 'Description'),
            'latitude' => Yii::t('app', 'Latitude'),
            'longitude' => Yii::t('app', 'Longitude'),
            'phone' => Yii::t('app', 'Phone'),
            'email' => Yii::t('app', 'Email'),
            'skype' => Yii::t('app', 'Skype'),
            'rating' => Yii::t('app', 'Rating'),
        );
    }

    /**
     * Retrieves a list of models based on the current search/filter conditions.
     * 
     * @return CActiveDataProvider the data provider that can return the models based on the search/filter conditions.
     */
    public function search()
    {
        $criteria = new CDbCriteria;

        $criteria->compare('id', $this->id);
        
        $criteria->compare('google_id', $this->google_id);

        $criteria->compare('name', $this->name, true);

        $criteria->compare('description', $this->description, true);

        $criteria->compare('latitude', $this->latitude);

        $criteria->compare('longitude', $this->longitude);

        $criteria->compare('phone', $this->phone, true);

        $criteria->compare('email', $this->email, true);

        $criteria->compare('skype', $this->skype, true);
        
        $criteria->compare('rating', $this->rating);

        return new CActiveDataProvider('Poi', array(
            'criteria' => $criteria,
        ));
    }
    
    /**
     * Getting some attributes with user defined functions
     * 
     * @param string $name property name
     * 
     * @return mixed property value
     * @see CActiveRecord::__get()
     */
    public function __get($name)
    {
        if ($name == 'rating') {
            return $this->getRating();
        }
        return parent::__get($name);
    }
    
    /**
     * Returns poi rating value
     * 
     * @return integer poi rating value
     */
    public function getRating()
    {
        if (!$this->isNewRecord) {
            $model = new PoiRating;
            $model->attributes = Yii::app()->params['rating']['poi'];
            $model->poi = $this;
            $value = $model->getValue();
            return $value;
        }
        return $this->getAttribute('rating');
    }
    
    /**
     * This method is invoked before deleting a record
     * 
     * @return boolean whether the record should be deleted. Defaults to true
     * @see CActiveRecord::beforeDelete()
     */
    protected function beforeDelete()
    {
        foreach ($this->poi_categories as $categor) {
            if (!$categor->delete())
                return  false;
        }
        return parent::beforeDelete();
    }
}