<?php
/**
 * PoiApiModel class
 * 
 * PHP version 5
 *  
 * @category ADPOI
 * @package  App.model
 * @author   Pohil Alexandr <pohil@5-soft.com>, Evgeniy Marilev <jeka@5-soft.com>
 * @license  http://adpoi.com/display/license proprietary
 * @link     http://adpoi.com/display/PhpDoc
 */
/**
 * PoiApiModel class
 *  * 
 * PoiApiModel is the model class for poi
 *  
 * @category ADPOI
 * @package  App.model
 * @author   Pohil Alexandr <pohil@5-soft.com>, Evgeniy Marilev <jeka@5-soft.com>
 * @license  http://adpoi.com/display/license proprietary
 * @link     http://adpoi.com/display/PhpDoc
 */
class PoiApiModel extends RestApiModel
{
    /**
     * Poi's id
     * @var integer
     */
    public $id;
    
    /**
     * User id
     * @var string
     */
    public $user_id;
    
    /**
     * Poi's name
     * @var string
     */
    public $name;

    /**
     * Poi's description
     * @var string
     */
    public $description;
    
    /**
     * Poi's latitude
     * @var double
     */
    public $latitude;
    
    /**
     * Poi's longitude
     * @var double
     */
    public $longitude;
    
    /**
     * Poi's phone
     * @var sting
     */
    public $phone;
    
    /**
     * Poi's email
     * @var sting
     */
    public $email;
    
    /**
     * Skype of poi's manager
     * @var sting
     */
    public $skype;
    
    /**
     * Category name or comma separated categories names
     * @var string
     */
    public  $category;
    
    /**
     * Priority name or comma separated priorities names
     * @var string
     */
    public $priority;
    
    /**
     * Number of items per page
     * @var int
     */
    public $count;
    
    /**
     * Page number
     * @var int
     */
    public $page;
    
    /**
     * Seach radius
     * @var int
     */
    public $radius;
    
    /**
     * Search term
     * @var string
     */
    public $term;
    
    /**
     * Return the validation rulles
     * 
     * @return array validation rules to be applied 
     * when {@link validate()} is called.
     * @see CModel::rules()
     */
    public function rules()
    {
        $base = parent::rules();
        $rules = array(
            array('id', 'required', 'on' => 'update, delete, show'),
            array('id', 'numerical', 'integerOnly' => true, 'on' => 'update, delete, show'),
            array('id', 'exist', 'className' => 'Poi', 'attributeName' => 'id', 'on' => 'update, delete, show'),
            array('name, latitude, longitude, category', 'required', 'on' => 'create'),
            array('name', 'length', 'max'=>128, 'on' => 'create, update'),
            array('category', 'length', 'max'=>256, 'on' => 'create, update, list'),
            array('priority', 'length', 'max'=>256, 'on' => 'list'),
            array('category', 'isCorrectCategories', 'on' => 'create, update, list', 'message' => Yii::t('api', 'Bad category value')),
            array('priority', 'isCorrectPriorities', 'on' => 'list', 'message' => Yii::t('api', 'Bad priority value')),
            array('latitude, ', 'numerical', 'min' => -90, 'max' => 90, 'on' => 'create, update, list'),
            array('longitude', 'numerical', 'min' => -180, 'max' => 180, 'on' => 'create, update, list'),
            array('phone, skype, email', 'length', 'max'=>128, 'allowEmpty' => false, 'on' => 'create, update'),
            array('email', 'email', 'on' => 'create, update'),
            array('description', 'safe', 'on' => 'create, update'),
            array('latitude, longitude, radius', 'required', 'on' => 'list'),
            array('term', 'length', 'max' => 64, 'on' => 'list'),
            array('page', 'default', 'value' => 1, 'on' => 'list'),
            array('count', 'default', 'value' => 5, 'on' => 'list'),
        );
        return CMap::mergeArray($base, $rules);
    }
    
    /**
     * Submits changes into database for create operations
     *  
     * @return bool is operation success 
     * @see RestApiModel::onCreate()
     */
    protected function onCreate()
    {
        $record = $this->loadRecord();
        $record->attributes = $this->attributes;
        $categoriesId = Category::getIdsByNames($this->getCategoriesNames());
        if (!$record->save(false)) {
            return false;
        }
        $isSuccess = true;
        foreach ($categoriesId as $category_id) {
            $poiCategory = new PoiCategory();
            $poiCategory->poi_id = $record->id;
            $poiCategory->category_id = $category_id;
            $isSuccess = $isSuccess && $poiCategory->save(false);
        }
        return $isSuccess;
    }
    
    /**
     * Submits changes into database for update operations
     *  
     * @return bool is operation success 
     * @see RestApiModel::onUpdate()
     */
    protected function onUpdate()
    {
        $record = $this->loadRecord();
        $record->attributes = $this->attributes;
        $oldPoiCategories = $record->poi_categories;
        if (!$record->save(false)) {
            return false;
        }
        if (isset($this->category)) {
            foreach ($oldPoiCategories as $poi_category) {
                $poi_category->delete();
            }
            $categoriesId = Category::getIdsByNames($this->getCategoriesNames());
            $isSuccess = true;
            foreach ($categoriesId as $category_id) {
                $poiCategory = new PoiCategory();
                $poiCategory->poi_id = $record->id;
                $poiCategory->category_id = $category_id;
                $isSuccess = $isSuccess && $poiCategory->save(false);
            }
            return $isSuccess;
        }
        return true;
    }
    
    /**
     * Submits changes into database for delete operations
     *  
     * @return bool is operation success 
     * @see RestApiModel::onDelete()
     */
    protected function onDelete()
    {
        $record = $this->loadRecord();
        try {
            return $record->delete();
        } catch (CDbException $e) {
            $message = Yii::t('api', 'This poi connected with other object');
            throw new RestHttpException(RestHttpResponse::ERROR_CODE_INTERNAL_SERVER_ERROR, $message);
        }
        return false;
    }
    
    
    /**
     * Fetchs api model's records list.
     * Used only for "list" API method
     * 
     * @return array records list
     */
    protected function fetchData()
    {
        $poiSeach = new PoiSearch();
        $poiSeach->latitude = $this->latitude;
        $poiSeach->longitude = $this->longitude;
        $poiSeach->radius = $this->radius;
        $poiSeach->term = $this->term;
        $poiSeach->category = $this->isCorrectCategories('category', array());
        $poiSeach->priority = $this->isCorrectPriorities('priority', array());
        $poiSeach->user_id = $this->user_id;
        $poiSeach->count = $this->count;
        $poiSeach->page = $this->page;
        $result = $poiSeach->search();
        return $result->getData();
    }
    
    /**
     * Validate function for checking category parameter
     * 
     * @param string $attribute variable name for validate
     * @param array  $params    additional parametrs for validate
     * 
     * @return array selected categories names
     */
    public function isCorrectCategories($attribute, $params)
    {
        $words = $this->getCategoriesNames();
        $diff = array_diff($words, PoiSearch::getAvailableCategories());
        if (!empty($diff)) {
            $this->addError($attribute, $params['message']);
        }
        return $words;
    }
    
    /**
     * Split the string of categories name to category word
     * 
     * @return array categoties name
     */
    public function getCategoriesNames()
    {
        $parser = new CQueryParser($this->category);
        $parser->separator = ',';
        $words = $parser->parse();
        return array_unique($words);
    }
    
    /**
     * Validate function for checking priority parameter
     * 
     * @param string $attribute variable name for validate
     * @param array  $params    additional parametrs for validate
     * 
     * @return array selected priorities names
     */
    public function isCorrectPriorities($attribute, $params)
    {
        $parser = new CQueryParser($this->priority);
        $parser->separator = ',';
        $words = $parser->parse();
        $diff = array_diff($words, PoiSearch::getAvailablePriorities());
        if (!empty($diff)) {
            $this->addError($attribute, $params['message']);
        }
        return $words;
    }
}
